@@ -572,10 +572,11 @@ class UniqueExitsGraph(nx.MultiDiGraph, Generic[Node, Edge]):
         self,
         edgeSep: str = '::',
         external: Optional[Set[Node]] = None,
-        explorationOrder: Tuple[Node, Sequence[Edge]] = None,
+        explorationOrder: Optional[Tuple[Node, Sequence[Edge]]] = None,
         edgeOrders: Union[
             Dict[Node, Sequence[Edge]],
-            Dict[Node, Dict[Edge, Any]]
+            Dict[Node, Dict[Edge, Any]],
+            None
         ] = None
     ):
         """
