@@ -10,10 +10,13 @@ from __future__ import annotations
 import argparse
 import pathlib
 import textwrap
+import sys
+import csv
 import networkx as nx  # type: ignore
 
 from typing import (
-    Literal, Optional, Union, get_args, TypeAlias
+    Literal, Optional, Union, get_args, TypeAlias, List, Callable, Dict,
+    Sequence
 )
 
 from . import journal
@@ -79,7 +82,16 @@ def loadDotFile(path: pathlib.Path) -> core.DecisionGraph:
     (whose extension should normally be '.dot'). The file format is the
     GraphViz "dot" format.
     """
-    raise NotImplementedError(":(")
+    with path.open('r', encoding='utf-8') as fInput:
+        dot = fInput.read()
+        try:
+            return core.DecisionGraph.fromDot(dot)
+        except core.DotParseError:
+            raise core.DotParseError(
+                "Failed to parse Dot file contents:\n\n"
+              + dot
+              + "\n\n(See error above for specific parsing issue.)"
+            )
 
 
 def saveDotFile(path: pathlib.Path, graph: core.DecisionGraph) -> None:
@@ -87,8 +99,9 @@ def saveDotFile(path: pathlib.Path, graph: core.DecisionGraph) -> None:
     Saves a `core.DecisionGraph` as a GraphViz "dot" file. The file
     extension should normally be ".dot".
     """
-    # TODO: Custom method for this to capture more stuff?
-    nx.nx_pydot.write_dot(graph, str(path))
+    dotStr = graph.toDot()
+    with path.open('w', encoding='utf-8') as fOutput:
+        fOutput.write(dotStr)
 
 
 def loadExploration(path: pathlib.Path) -> core.Exploration:
@@ -166,6 +179,80 @@ def loadSource(
         )
 
 
+#---------------------#
+# Analysis tool lists #
+#---------------------#
+
+AnalysisResult: TypeAlias = Union[None, bool, str, int, float, complex]
+"""
+A type alias for values we're willing to accept as analysis results.
+These are going to be written to a CSV file that we want to be
+human-readable.
+"""
+
+STEPWISE_DECISION_ANALYSIS_TOOLS: Dict[
+    str,
+    Callable[[core.Situation, core.Decision], AnalysisResult]
+] = {
+    "actionCount": analysis.analyzeGraph(analysis.countActionsAtDecision),
+    "branchCount": analysis.analyzeGraph(analysis.countBranches)
+}
+"""
+The analysis functions to apply to each decision in each step when
+analyzing an exploration, and the names for each.
+"""
+
+STEP_ANALYSIS_TOOLS: Dict[
+    str,
+    Callable[[core.Situation], AnalysisResult]
+] = {
+    "unexploredCount": analysis.countAllUnexploredBranches,
+    "traversableUnexploredCount": analysis.countTraversableUnexploredBranches,
+    "meanActions": analysis.meanOfResults(
+        analysis.perDecision(
+            analysis.analyzeGraph(analysis.countActionsAtDecision)
+        )
+    ),
+    "meanBranches": analysis.meanOfResults(
+        analysis.perDecision(
+            analysis.analyzeGraph(analysis.countBranches)
+        )
+    ),
+}
+"""
+The analysis functions to apply to each step when analyzing an
+exploration, and the names for each.
+"""
+
+DECISION_ANALYSIS_TOOLS: Dict[
+    str,
+    Callable[[core.Exploration, core.Decision], AnalysisResult]
+] = {
+    "isKnown": lambda e, d: d in e.allKnownDecisions(),
+    "revisitCount": analysis.countRevisits,
+}
+"""
+The analysis functions to apply once to each decision in an exploration,
+and the names for each.
+"""
+
+WHOLE_ANALYSIS_TOOLS: Dict[
+    str,
+    Callable[[core.Exploration], AnalysisResult]
+] = {
+    "meanRevisits": analysis.meanOfResults(
+        analysis.perExplorationDecision(
+            analysis.countRevisits,
+            mode="visited"
+        )
+    )
+}
+"""
+The analysis functions to apply to entire explorations, and the names
+for each.
+"""
+
+
 #---------------#
 # API Functions #
 #---------------#
@@ -200,7 +287,9 @@ def show(
 
 def analyze(
     source: pathlib.Path,
-    formatOverride: Optional[SourceType] = None
+    formatOverride: Optional[SourceType] = None,
+    destination: Optional[pathlib.Path] = None,
+    applyTools: Optional[List[str]] = None
 ) -> None:
     """
     Analyzes the exploration stored in the `source` file. The file
@@ -209,16 +298,113 @@ def analyze(
     are treated as JSON-encoded exploration objects, while '.exj' files
     are treated as journals using the default journal format.
 
-    TODO: What does this actually do?
+    This applies a number of analysis functions to produce a CSV file
+    showing per-decision-per-step, per-decision, per-step, and
+    per-exploration metrics. A subset of the available metrics may be
+    selected by passing a list of strings for the `applyTools` argument.
+    See the `STEPWISE_DECISION_ANALYSIS_TOOLS`, `STEP_ANALYSIS_TOOLS`,
+    `DECISION_ANALYSIS_TOOLS`, and `WHOLE_ANALYSIS_TOOLS` dictionaries
+    for tool names.
+
+    If no output file is specified, the output will be printed out.
     """
+    # Load our source exploration object:
     obj = loadSource(source, formatOverride)
     if isinstance(obj, core.DecisionGraph):
         obj = core.Exploration.fromGraph(obj)
 
-    print("Unexplored options at each step:")
-    print(*analysis.unexploredBranchesPerStep(obj))
+    exploration: core.Exploration = obj
 
-    # TODO: more here? Options for which analysis to apply?
+    # Apply all of the analysis functions (or only just those that are
+    # selected using applyTools):
+
+    wholeRows: List[List[AnalysisResult]] = [['Whole exploration metrics:']]
+    # One row per tool
+    for tool in WHOLE_ANALYSIS_TOOLS:
+        if (applyTools is None) or (tool in applyTools):
+            wholeRows.append(
+                [tool, WHOLE_ANALYSIS_TOOLS[tool](exploration)]
+            )
+
+    decisionRows: List[Sequence[AnalysisResult]] = [
+        ['Per-decision metrics:']
+    ]
+    # One row per tool; one column per decision
+    columns = ['Metric ↓/Decision →'] + exploration.allDecisions()
+    decisionRows.append(columns)
+    for tool in DECISION_ANALYSIS_TOOLS:
+        if (applyTools is None) or (tool in applyTools):
+            row: List[AnalysisResult] = [tool]
+            decisionRows.append(row)
+            for decision in columns[1:]:
+                row.append(
+                    DECISION_ANALYSIS_TOOLS[tool](exploration, decision)
+                )
+
+    stepRows: List[Sequence[AnalysisResult]] = [
+        ['Per-step metrics:']
+    ]
+    # One row per exploration step; one column per tool
+    columns = ['Step ↓/Metric →']
+    stepRows.append(columns)
+    for i, situation in enumerate(exploration):
+        row = [i]
+        stepRows.append(row)
+        for tool in STEP_ANALYSIS_TOOLS:
+            if (applyTools is None) or (tool in applyTools):
+                if i == 0:
+                    columns.append(tool)
+                row.append(STEP_ANALYSIS_TOOLS[tool](situation))
+
+    stepwiseRows: List[Sequence[AnalysisResult]] = [
+        ['Per-decision-per-step, metrics (one table per metric):']
+    ]
+    # For each tool; one row per exploration step and one column per
+    # decision
+    columns = ['Step ↓/Decision →'] + exploration.allDecisions()
+    for tool in STEPWISE_DECISION_ANALYSIS_TOOLS:
+        if (applyTools is None) or (tool in applyTools):
+            stepwiseRows.append([tool])
+            stepwiseRows.append(columns)
+            for i, situation in enumerate(exploration):
+                row = [i]
+                stepwiseRows.append(row)
+                for decision in columns[1:]:
+                    row.append(
+                        STEPWISE_DECISION_ANALYSIS_TOOLS[tool](
+                            situation,
+                            decision
+                        )
+                    )
+
+    # Build a grid containing just the non-empty analysis categories, so
+    # that if you deselect some tools you get a smaller CSV file:
+    grid: List[Sequence[AnalysisResult]] = []
+    if len(wholeRows) > 1:
+        grid.extend(wholeRows)
+    for block in decisionRows, stepRows, stepwiseRows:
+        if len(block) > 1:
+            if grid:
+                grid.append([])  # spacer
+            grid.extend(block)
+
+    # Figure out our destination stream:
+    if destination is None:
+        outStream = sys.stdout
+        closeIt = False
+    else:
+        outStream = open(destination, 'w')
+        closeIt = True
+
+    # Create a CSV writer for our stream
+    writer = csv.writer(outStream)
+
+    # Write out our grid to the file
+    try:
+        writer.writerows(grid)
+    finally:
+        if closeIt:
+            outStream.close()
 
 
 def convert(
@@ -235,8 +421,7 @@ def convert(
     are:
 
     - '.dcg' A `core.DecisionGraph` stored in JSON format.
-    - '.dot' A `core.DecisionGraph` stored as a GraphViz DOT file (TODO:
-        reading this format).
+    - '.dot' A `core.DecisionGraph` stored as a GraphViz DOT file.
     - '.exp' A `core.Exploration` stored in JSON format.
     - '.exj' A `core.Exploration` stored as a journal (see
         `journal.JournalObserver`; TODO: writing this format).
@@ -247,8 +432,7 @@ def convert(
     decision graph format, only the current graph will be saved, unless
     `--step` is used to specify a different step index to save.
     """
-    # TODO dot reading, journal writing, and JSON reading/writing for
-    # all objects...
+    # TODO journal writing
     obj = loadSource(source, inputFormatOverride)
 
     if outputFormatOverride is None:
@@ -328,6 +512,15 @@ analyzeParser.add_argument(
     type=pathlib.Path,
     help="The file holding the exploration to analyze"
 )
+analyzeParser.add_argument(
+    "destination",
+    default=None,
+    type=pathlib.Path,
+    help=(
+        "The file name where the output should be written (this file"
+        " will be overwritten without warning)."
+    )
+)
 analyzeParser.add_argument(
     '-f',
     "--format",
@@ -397,7 +590,8 @@ if __name__ == "__main__":
     elif options.run == "analyze":
         analyze(
             options.source,
-            formatOverride=options.format
+            formatOverride=options.format,
+            destination=options.destination
         )
     elif options.run == "convert":
         convert(
