@@ -108,15 +108,19 @@ The program will prompt interactively for information when arguments are
 omitted.
 '''
     )
-    parser.add_argument('command', help='''\
+    parser.add_argument(
+        '-c',
+        '--command',
+        help='''\
 Which command to run:
     - 'display' for displaying a graph using matplotlib,
     - 'convert' for converting between graph/journal formats, or
     - 'analyze' for analyzing a graph'''
-    )  # noqa
-    parser.add_argument('inputFile', help='The input filename.')
+    )
+    parser.add_argument('-i', '--inputFile', help='The input filename.')
     parser.add_argument(
-        'outputFile',
+        '-o',
+        '--outputFile',
         help="The output filename, when using 'convert' or 'analyze'."
     )
     parsed = parser.parse_args()
@@ -165,7 +169,12 @@ What would you like to do? (enter a number; default is 0) """)
         exploration.main.show(source, formatOverride=sourceType)
 
     if command.strip() == 'analyze':
-        exploration.main.analyze(source, formatOverride=sourceType)
+        destination = getattr(parsed, 'outputFile', None)
+        exploration.main.analyze(
+            source,
+            formatOverride=sourceType,
+            destination=destination
+        )
 
     elif command.strip() == "convert":
         outputFile = parsed.outputFile
